#include <visa.h>
#include <ansi_c.h>
#include <utility.h>
#include <cvirte.h>     
#include <userint.h>
#include "rssmu.h"
#include "RSSMU_GPS_Example.h"


/* Error check macro for driver functions */

#define CHECKERR( fCal ) \
    { \
        error = (fCal);\
        rssmu_error_message (io, error, error_message);\
        if (error < VI_SUCCESS) {\
            sprintf (error_buffer, "Primary Error: 0x%08X, %s\n", error, error_message);\
            MessagePopup ("Error", error_buffer); \
            rssmu_error_query (io, &error, error_message);\
            sprintf (error_buffer, "Secondary Error: 0x%08X, %s\n", error, error_message);\
            SetWaitCursor (0); \
            MessagePopup ("Error", error_buffer); \
            rssmu_close(io); \
            SetCtrlAttribute (panelHandle, PANEL_APPLY, ATTR_DIMMED, 0);\
            SetCtrlAttribute (panelHandle, PANEL_Exit, ATTR_DIMMED, 0);\
            break; \
        } \
    }
    
/* Globals */

ViInt32 panelHandle;
ViSession io;
ViStatus    error;
ViChar      error_message[256];
ViChar      error_buffer[256];

/* Local Functions */
int initialize (void);

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)
        return -1;  /* out of memory */
    if ((panelHandle = LoadPanel (0, "RSSMU_GPS_Example.uir", PANEL)) < 0)
        return -1;
    DisplayPanel (panelHandle);
    RunUserInterface ();
    DiscardPanel (panelHandle);
    return 0;
}

/*===========================================================================*/
/* Function: Exit                                                            */
/* Purpose:  This is a callback function of Exit button. It closes the       */
/*           session and exits the example.                                  */
/*===========================================================================*/
int CVICALLBACK exitApp (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
        {
        case EVENT_COMMIT:
            QuitUserInterface (0);
            break;
        }
    return 0;
}

/*===========================================================================*/
/* Function: Apply Settings                                                  */
/* Purpose:  This is a callback function of Apply button. The values are     */
/*           send to the instrument                                          */
/*===========================================================================*/
int CVICALLBACK apllySettings (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt32     id, data_source, time_shift, sync;
    ViBoolean   state, gps_state;
    ViReal64    rf_level, rf_freq, power, doppler;

    switch (event)
        {
        case EVENT_COMMIT:
             SetCtrlAttribute (panelHandle, PANEL_APPLY, ATTR_DIMMED, 1);
             SetCtrlAttribute (panelHandle, PANEL_Exit, ATTR_DIMMED, 1);
             
             ProcessDrawEvents ();
             
             if (initialize() < VI_SUCCESS) break;

             SetWaitCursor (1);

             GetCtrlVal (panelHandle, PANEL_GPS_STATE, &gps_state);
             GetCtrlVal (panelHandle, PANEL_RF_STATE, &state);
             
             //For Synchronisation 
             //Enable Operation Complete for STB Register
             CHECKERR (rssmu_EventStatusEnable(io,1));
             //Clear Status
             CHECKERR (rssmu_ClearStatus(io));
             
             // wait for complet switch to GPS subsystem
             do
             {
                Delay (0.1);
                CHECKERR (rssmu_getRegister (io, RSSMU_REGISTER_OPERATION, RSSMU_REGISTER_COND,&sync));
                ProcessSystemEvents ();
             } while ((sync & 2) != 0);

             GetCtrlVal (panelHandle, PANEL_SAT_DOPPLER, &doppler);
             GetCtrlVal (panelHandle, PANEL_SAT_POWER, &power);
             GetCtrlVal (panelHandle, PANEL_SAT_ID, &id);
             GetCtrlVal (panelHandle, PANEL_SAT_TIMESHIFT, &time_shift);
             GetCtrlVal (panelHandle, PANEL_DATA_SOURCE, &data_source);
             CHECKERR (rssmu_SetGPSDataSource (io, RSSMU_PATH_A, data_source));

             CHECKERR (rssmu_ConfigureGPSSatellite (io, RSSMU_PATH_A, 1, VI_TRUE, id,
                                                    RSSMU_GPS_RANGING_CODE_CA, time_shift,
                                                    power, doppler));
             GetCtrlVal (panelHandle, PANEL_RF_LEVEL, &rf_level);
             GetCtrlVal (panelHandle, PANEL_RF_FREQ, &rf_freq); 
             CHECKERR (rssmu_SetRFAmplitude (io, RSSMU_PATH_A, rf_level));
             CHECKERR (rssmu_SetRFFrequency (io, RSSMU_PATH_A, rf_freq));
             
             CHECKERR (rssmu_SetGPSState (io, RSSMU_PATH_A, gps_state));

             CHECKERR (rssmu_SetOutputState (io, RSSMU_PATH_A, state));

             rssmu_close(io);

             SetWaitCursor (0);
             SetCtrlAttribute (panelHandle, PANEL_APPLY, ATTR_DIMMED, 0);
             SetCtrlAttribute (panelHandle, PANEL_Exit, ATTR_DIMMED, 0);
            break;
        }
    return 0;
}

/*===========================================================================*/
/* Function: Panel Help                                                      */
/* Purpose:  This is a callback function of the menu bar. It displays a help */
/*           window describing panel being used.                             */
/*===========================================================================*/
void CVICALLBACK panelHelp (int menuBar, int menuItem, void *callbackData,
        int panel)
{

    if (panel == panelHandle)
        MessagePopup ("Help", "This is a simple demo program using\n"
                              "the Rohde&Schwarz SMU Vector Signal Generator\n"
                              "instrument driver provided in the package.\n");

    return;
}

/*===========================================================================*/ 
/* Local Functions Definitions                                               */
/*===========================================================================*/

/*===========================================================================*/
/* Function: Initialize                                                      */
/* Purpose:  This function initializes the instrument, performs ID Query     */
/*           and Reset                                                       */
/*===========================================================================*/
int initialize (void)
{
    ViChar      instr_desc[256];
    ViBoolean   id;
    ViBoolean   rst;
    

    SetWaitCursor(0);
    GetCtrlVal (panelHandle, PANEL_INSTR_DESC, instr_desc);
    GetCtrlVal (panelHandle, PANEL_IDQUERY, &id);
    GetCtrlVal (panelHandle, PANEL_RESET, &rst);
    SetWaitCursor(1);
    if ((error = rssmu_init (instr_desc, id, rst, &io)) < 0 )
    {
        rssmu_error_message(VI_NULL, error, error_message);
        sprintf (error_buffer, "Initialization Error:\n\n%s\n\nCheck your "
                "connections and make sure you have the right instrument descriptor.",
                 error_message);
        MessagePopup ("ERROR!", error_buffer);
        SetWaitCursor(0);
        return(-1);   
    }

    viSetAttribute (io, VI_ATTR_TMO_VALUE, 10000);
    SetWaitCursor(0);
    return 0;
}
